@@ -20,9 +20,12 @@ L{CompositeConfigParser} usage:
 >>> parser = ArgumentParser(..., default_config_files=['./pyproject.toml', 'setup.cfg', 'my_super_tool.ini'], config_file_parser_class=MixedParser)
 
 """
+from __future__ import annotations
+
 import argparse
 from collections import OrderedDict
 import re
+import sys
 from typing import Any, Callable, Dict, List, Optional, Tuple, TextIO, Union
 import csv
 import functools
@@ -31,7 +34,17 @@ from ast import literal_eval
 import warnings
 
 from configargparse import ConfigFileParserException, ConfigFileParser, ArgumentParser
-import toml
+
+if sys.version_info >= (3, 11):
+    from tomllib import load as _toml_load
+    import io
+    # The tomllib module from the standard library 
+    # expect a binary IO and will fail if receives otherwise. 
+    # So we hack a compat function that will work with TextIO and assume the utf-8 encoding.
+    def toml_load(stream: TextIO) -> Any:
+        return _toml_load(io.BytesIO(stream.read().encode()))
+else:
+    from toml import load as toml_load
 
 # I did not invented these regex, just put together some stuff from:
 # - https://stackoverflow.com/questions/11859442/how-to-match-string-in-quotes-using-regex
@@ -161,7 +174,7 @@ class TomlConfigParser(ConfigFileParser):
         """Parses the keys and values from a TOML config file."""
         # parse with configparser to allow multi-line values
         try:
-            config = toml.load(stream)
+            config = toml_load(stream)
         except Exception as e:
             raise ConfigFileParserException("Couldn't parse TOML file: %s" % e)
 
