@@ -4,7 +4,6 @@
 # Author: Edward Loper <edloper@loper.org>
 # URL: <http://epydoc.sf.net>
 #
-
 """
 Syntax highlighter for Python values.  Currently provides special
 colorization support for:
@@ -32,6 +31,7 @@ C{__repr__}.
 B{Usage}: 
 >>> 
 """
+from __future__ import annotations
 
 __docformat__ = 'epytext en'
 
@@ -43,7 +43,6 @@ from inspect import signature
 from typing import Any, AnyStr, Union, Callable, Dict, Iterable, Sequence, Optional, List, Tuple, cast
 
 import attr
-import astor.op_util
 from docutils import nodes
 from twisted.web.template import Tag
 
@@ -51,7 +50,7 @@ from pydoctor.epydoc import sre_parse36, sre_constants36 as sre_constants
 from pydoctor.epydoc.markup import DocstringLinker
 from pydoctor.epydoc.markup.restructuredtext import ParsedRstDocstring
 from pydoctor.epydoc.docutils import set_node_attributes, wbr, obj_reference, new_document
-from pydoctor.astutils import node2dottedname, bind_args, Parentage, get_parents
+from pydoctor.astutils import node2dottedname, bind_args, Parentage, get_parents, unparse, op_util
 
 def decode_with_backslashreplace(s: bytes) -> str:
     r"""
@@ -76,6 +75,7 @@ class _MarkedColorizerState:
     charpos: int
     lineno: int
     linebreakok: bool
+    stacklength: int
 
 class _ColorizerState:
     """
@@ -87,18 +87,20 @@ class _ColorizerState:
     then fall back on a multi-line output if that fails.  
     """
     def __init__(self) -> None:
-        self.result: List[nodes.Node] = []
+        self.result: list[nodes.Node] = []
         self.charpos = 0
         self.lineno = 1
         self.linebreakok = True
-        self.warnings: List[str] = []
+        self.warnings: list[str] = []
+        self.stack: list[ast.AST] = []
 
     def mark(self) -> _MarkedColorizerState:
         return _MarkedColorizerState(
                     length=len(self.result), 
                     charpos=self.charpos,
                     lineno=self.lineno, 
-                    linebreakok=self.linebreakok)
+                    linebreakok=self.linebreakok,
+                    stacklength=len(self.stack))
 
     def restore(self, mark: _MarkedColorizerState) -> List[nodes.Node]:
         """
@@ -109,16 +111,17 @@ class _ColorizerState:
                                         mark.linebreakok)
         trimmed = self.result[mark.length:]
         del self.result[mark.length:]
+        del self.stack[mark.stacklength:]
         return trimmed
 
 # TODO: add support for comparators when needed. 
 # _OperatorDelimitier is needed for:
-# - IfExp
-# - UnaryOp
-# - BinOp, needs special handling for power operator
-# - Compare
-# - BoolOp
-# - Lambda
+# - IfExp (TODO)
+# - UnaryOp (DONE)
+# - BinOp, needs special handling for power operator (DONE)
+# - Compare (TODO)
+# - BoolOp (DONE)
+# - Lambda (TODO)
 class _OperatorDelimiter:
     """
     A context manager that can add enclosing delimiters to nested operators when needed. 
@@ -127,7 +130,7 @@ class _OperatorDelimiter:
     """
 
     def __init__(self, colorizer: 'PyvalColorizer', state: _ColorizerState, 
-                 node: Union[ast.UnaryOp, ast.BinOp, ast.BoolOp],) -> None:
+                 node: ast.expr,) -> None:
 
         self.discard = True
         """No parenthesis by default."""
@@ -145,17 +148,21 @@ class _OperatorDelimiter:
         
         # avoid needless parenthesis, since we now collect parents for every nodes 
         if isinstance(parent_node, (ast.expr, ast.keyword, ast.comprehension)):
-            precedence = astor.op_util.get_op_precedence(node.op)
-            if isinstance(parent_node, (ast.UnaryOp, ast.BinOp, ast.BoolOp)):
-                parent_precedence = astor.op_util.get_op_precedence(parent_node.op)
-                if isinstance(parent_node.op, ast.Pow) or isinstance(parent_node, ast.BoolOp):
-                    parent_precedence+=1
-            else:
-                parent_precedence = colorizer.explicit_precedence.get(
-                    node, astor.op_util.Precedence.highest)
-                
-            if precedence < parent_precedence:
+            try:
+                precedence = op_util.get_op_precedence(getattr(node, 'op', node))
+            except KeyError:
                 self.discard = False
+            else:
+                try:
+                    parent_precedence = op_util.get_op_precedence(getattr(parent_node, 'op', parent_node))
+                    if isinstance(getattr(parent_node, 'op', None), ast.Pow) or isinstance(parent_node, ast.BoolOp):
+                        parent_precedence+=1
+                except KeyError:
+                    parent_precedence = colorizer.explicit_precedence.get(
+                        node, op_util.Precedence.highest)
+                    
+                if precedence < parent_precedence:
+                    self.discard = False
 
     def __enter__(self) -> '_OperatorDelimiter':
         return self
@@ -383,24 +390,25 @@ class PyvalColorizer:
         while num_chars > 0:
             if not result: 
                 return
-            if isinstance(result[-1], nodes.Element):
-                if len(result[-1].children) >= 1:
-                    data = result[-1][-1].astext()
+            if isinstance(r1:=result[-1], nodes.Element):
+                if len(r1.children) >= 1:
+                    data = r1[-1].astext()
                     trim = min(num_chars, len(data))
-                    result[-1][-1] = nodes.Text(data[:-trim])
-                    if not result[-1][-1].astext(): 
-                        if len(result[-1].children) == 1:
+                    r1[-1] = nodes.Text(data[:-trim])
+                    if not r1[-1].astext(): 
+                        if len(r1.children) == 1:
                             result.pop()
                         else:
-                            result[-1].pop()
+                            r1.pop()
                 else:
                     trim = 0
                     result.pop()
                 num_chars -= trim
             else:
                 # Must be Text if it's not an Element
-                trim = min(num_chars, len(result[-1]))
-                result[-1] = nodes.Text(result[-1].astext()[:-trim])
+                assert isinstance(r1, nodes.Text)
+                trim = min(num_chars, len(r1))
+                result[-1] = nodes.Text(r1.astext()[:-trim])
                 if not result[-1].astext(): 
                     result.pop()
                 num_chars -= trim
@@ -460,7 +468,7 @@ class PyvalColorizer:
                 self._insert_comma(indent, state)
             state.result.append(self.WORD_BREAK_OPPORTUNITY)
             if key:
-                self._set_precedence(astor.op_util.Precedence.Comma, val)
+                self._set_precedence(op_util.Precedence.Comma, val)
                 self._colorize(key, state)
                 self._output(': ', self.COLON_TAG, state)
             else:
@@ -512,19 +520,29 @@ class PyvalColorizer:
 
     @staticmethod
     def _is_ast_constant(node: ast.AST) -> bool:
-        return isinstance(node, (ast.Num, ast.Str, ast.Bytes, 
-                                 ast.Constant, ast.NameConstant, ast.Ellipsis))
+        if sys.version_info[:2] >= (3, 8):
+            return isinstance(node, ast.Constant)
+        else:
+            # TODO: remove me when python3.7 is not supported anymore
+            return isinstance(node, (ast.Num, ast.Str, ast.Bytes, 
+                    ast.Constant, ast.NameConstant, ast.Ellipsis))
     @staticmethod
     def _get_ast_constant_val(node: ast.AST) -> Any:
         # Deprecated since version 3.8: Replaced by Constant
-        if isinstance(node, ast.Num): 
-            return(node.n)
-        if isinstance(node, (ast.Str, ast.Bytes)):
-           return(node.s)
-        if isinstance(node, (ast.Constant, ast.NameConstant)):
-            return(node.value)
-        if isinstance(node, ast.Ellipsis):
-            return(...)
+        if sys.version_info[:2] >= (3, 8):
+            if isinstance(node, ast.Constant):
+                return node.value
+        else:
+            # TODO: remove me when python3.7 is not supported anymore
+            if isinstance(node, ast.Num): 
+                return(node.n)
+            if isinstance(node, (ast.Str, ast.Bytes)):
+                return(node.s)
+            if isinstance(node, (ast.Constant, ast.NameConstant)):
+                return(node.value)
+            if isinstance(node, ast.Ellipsis):
+                return(...)
+        raise RuntimeError(f'expected a constant: {ast.dump(node)}')
         
     def _colorize_ast_constant(self, pyval: ast.AST, state: _ColorizerState) -> None:
         val = self._get_ast_constant_val(pyval)
@@ -535,6 +553,7 @@ class PyvalColorizer:
             self._output('...', self.ELLIPSIS_TAG, state)
 
     def _colorize_ast(self, pyval: ast.AST, state: _ColorizerState) -> None:
+        state.stack.append(pyval)
         # Set nodes parent in order to check theirs precedences and add delimiters when needed.
         try:
             next(get_parents(pyval))
@@ -578,6 +597,7 @@ class PyvalColorizer:
             self._colorize_ast(pyval.value, state)
         else:
             self._colorize_ast_generic(pyval, state)
+        assert state.stack.pop() is pyval
     
     def _colorize_ast_unary_op(self, pyval: ast.UnaryOp, state: _ColorizerState) -> None:
         with _OperatorDelimiter(self, state, pyval):
@@ -598,38 +618,16 @@ class PyvalColorizer:
     def _colorize_ast_binary_op(self, pyval: ast.BinOp, state: _ColorizerState) -> None:
         with _OperatorDelimiter(self, state, pyval):
             # Colorize first operand
+            mark = state.mark()
             self._colorize(pyval.left, state)
-
             # Colorize operator
-            if isinstance(pyval.op, ast.Sub):
-                self._output('-', None, state)
-            elif isinstance(pyval.op, ast.Add):
-                self._output('+', None, state)
-            elif isinstance(pyval.op, ast.Mult):
-                self._output('*', None, state)
-            elif isinstance(pyval.op, ast.Div):
-                self._output('/', None, state)
-            elif isinstance(pyval.op, ast.FloorDiv):
-                self._output('//', None, state)
-            elif isinstance(pyval.op, ast.Mod):
-                self._output('%', None, state)
-            elif isinstance(pyval.op, ast.Pow):
-                self._output('**', None, state)
-            elif isinstance(pyval.op, ast.LShift):
-                self._output('<<', None, state)
-            elif isinstance(pyval.op, ast.RShift):
-                self._output('>>', None, state)
-            elif isinstance(pyval.op, ast.BitOr):
-                self._output('|', None, state)
-            elif isinstance(pyval.op, ast.BitXor):
-                self._output('^', None, state)
-            elif isinstance(pyval.op, ast.BitAnd):
-                self._output('&', None, state)
-            elif isinstance(pyval.op, ast.MatMult):
-                self._output('@', None, state)
-            else:
+            try:
+                self._output(op_util.get_op_symbol(pyval.op, ' %s '), None, state)
+            except KeyError:
                 state.warnings.append(f"Unknow binary operator: {pyval}")
+                state.restore(mark)
                 self._colorize_ast_generic(pyval, state)
+                return
 
             # Colorize second operand
             self._colorize(pyval.right, state)
@@ -672,6 +670,8 @@ class PyvalColorizer:
             # In Python < 3.9, non-slices are always wrapped in an Index node.
             sub = sub.value
         self._output('[', self.GROUP_TAG, state)
+        self._set_precedence(op_util.Precedence.Subscript, node)
+        self._set_precedence(op_util.Precedence.Index, sub)
         if isinstance(sub, ast.Tuple):
             self._multiline(self._colorize_iter, sub.elts, state)
         else:
@@ -751,7 +751,13 @@ class PyvalColorizer:
 
     def _colorize_ast_generic(self, pyval: ast.AST, state: _ColorizerState) -> None:
         try:
-            source = astor.to_source(pyval).strip()
+            # Always wrap the expression inside parenthesis because we can't be sure 
+            # if there are required since we don;t have support for all operators 
+            # See TODO comment in _OperatorDelimiter.
+            source = unparse(pyval).strip()
+            if sys.version_info > (3,9) and isinstance(pyval, 
+                    (ast.IfExp, ast.Compare, ast.Lambda)) and len(state.stack)>1:
+                source = f'({source})'
         except Exception: #  No defined handler for node of type <type>
             state.result.append(self.UNKNOWN_REPR)
         else:
@@ -1007,6 +1013,7 @@ class PyvalColorizer:
             # If the segment fits on the current line, then just call
             # markup to tag it, and store the result.
             # Don't break links into separate segments, neither quotes.
+            element: nodes.Node
             if (self.linelen is None or 
                 state.charpos + segment_len <= self.linelen 
                 or link is True 
@@ -1043,4 +1050,3 @@ class PyvalColorizer:
                 else:
                     element = nodes.Text(segment)
                 state.result += [element, self.LINEWRAP]
-	
