@@ -4,7 +4,6 @@
 #
 # Created [04/10/01 12:00 AM]
 #
-
 """
 Parser for epytext strings.  Epytext is a lightweight markup whose
 primary intended application is Python documentation strings.  This
@@ -122,6 +121,7 @@ Description::
 """
 # Note: the symbol list is appended to the docstring automatically,
 # below.
+from __future__ import annotations
 
 __docformat__ = 'epytext en'
 
@@ -139,9 +139,8 @@ import unicodedata
 from docutils import nodes
 from twisted.web.template import Tag
 
-from pydoctor.epydoc.markup import Field, ParseError, ParsedDocstring, ParserFunction
+from pydoctor.epydoc.markup import Field, ObjClass, ParseError, ParsedDocstring, ParserFunction
 from pydoctor.epydoc.docutils import set_node_attributes, new_document
-from pydoctor.model import Documentable
 
 ##################################################
 ## Helper functions
@@ -1297,7 +1296,7 @@ def parse_docstring(docstring: str, errors: List[ParseError]) -> ParsedDocstring
     else:
         return ParsedEpytextDocstring(None, fields)
 
-def get_parser(obj: Optional[Documentable]) -> ParserFunction:
+def get_parser(_: ObjClass | None) -> ParserFunction:
     """
     Get the L{parse_docstring} function. 
     """
@@ -1425,7 +1424,9 @@ class ParsedEpytextDocstring(ParsedDocstring):
             yield set_node_attributes(nodes.inline('', ''), document=self._document, children=variables)
         elif tree.tag == 'target':
             value, = variables
-            yield set_node_attributes(nodes.Text(value), document=self._document)
+            if not isinstance(value, nodes.Text):
+                raise AssertionError("target contents must be a simple text.")
+            yield set_node_attributes(value, document=self._document)
         elif tree.tag == 'italic':
             yield set_node_attributes(nodes.emphasis('', ''), document=self._document, children=variables)
         elif tree.tag == 'math':
@@ -1445,7 +1446,9 @@ class ParsedEpytextDocstring(ParsedDocstring):
         elif tree.tag == 'literalblock':
             yield set_node_attributes(nodes.literal_block('', ''), document=self._document, children=variables)
         elif tree.tag == 'doctestblock':
-            yield set_node_attributes(nodes.doctest_block(tree.children[0], tree.children[0]), document=self._document)
+            if not isinstance(contents:=tree.children[0], str):
+                raise AssertionError("doctest block contents is not a string")
+            yield set_node_attributes(nodes.doctest_block(contents, contents), document=self._document)
         elif tree.tag in ('fieldlist', 'tag', 'arg'):
             raise AssertionError("There should not be any field lists left")
         elif tree.tag == 'section':
