@@ -110,7 +110,7 @@ def test_generate_empty_functional() -> None:
     @contextmanager
     def openFileForWriting(path: str) -> Iterator[io.BytesIO]:
         yield output
-    inv_writer._openFileForWriting = openFileForWriting # type: ignore[assignment]
+    inv_writer._openFileForWriting = openFileForWriting # type: ignore
 
     inv_writer.generate(subjects=[], basepath='base-path')
 
@@ -595,6 +595,7 @@ class TestIntersphinxCache:
             def send(
                     self: requests.adapters.HTTPAdapter,
                     request: requests.PreparedRequest,
+                    *args:object,
                     **kwargs: object
                     ) -> requests.Response:
                 response: requests.Response
